AfterModelCallResult

Enum AfterModelCallResult 

pub enum AfterModelCallResult {
    Continue(LlmResponse),
}
Result from an after_model_call hook invocation.

After-model hooks can only continue with a (possibly modified) response. Short-circuiting is not supported for after-hooks since the model call has already completed.

Examples

use adk_plugin::AfterModelCallResult;
use adk_core::LlmResponse;

// Pass through the model response unchanged
let response = LlmResponse::default();
let result = AfterModelCallResult::Continue(response);

Variants



Continue(LlmResponse)

Continue with (possibly modified) LLM response.

The contained response will be passed to the next plugin in the chain, and ultimately returned to the agent as the model output.